/*
 *  Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of vdc-utils
 */

#pragma once

#include <vector>
#include <memory>
#include <functional>
#include "error.hpp"

namespace vdcutils {

  template<class Tcomm, class Tcb>
  class ApiQueue;

  template<class Tcomm, class Tcb>
  using ApiQueuePtr = std::shared_ptr<ApiQueue<Tcomm, Tcb>>;

  template<class Tcomm, class Tcb>
  class ApiQueue : public std::enable_shared_from_this<ApiQueue<Tcomm, Tcb>>
  {
    using ApiQueueStepCB = std::function<p44::ErrorPtr (Tcb aResult, p44::ErrorPtr aError)> ;
    using ApiQueueCB = std::function<void (p44::ErrorPtr aError)>;
    using ApiQueueStep = std::function<void()>;
    p44::ErrorPtr errorStep;
    Tcomm comm;
    ApiQueueCB finishCB;
    std::vector<ApiQueueStep> stepsVec;
    std::vector<ApiQueueStep>::iterator stepIter;

    void next_step(){
      if( !p44::Error::isOK( errorStep ) ){
        LOG(LOG_DEBUG, "ApiQueue finished with error %s", errorStep->description().c_str());
        finishCB(errorStep);
        return;
      }
      if( ++stepIter >= stepsVec.end()){
        stepsVec.clear();
        finishCB(p44::Error::ok());
        return ;
      }
      (*stepIter)();
    }

  public:
    explicit ApiQueue(Tcomm aComm) :
      comm(aComm),
      finishCB([=](auto...){}),
      stepsVec({}),
      stepIter(stepsVec.begin())
      {}

    ApiQueuePtr<Tcomm, Tcb> getptr() {
        return this->shared_from_this();
    }

    void add(auto command, ApiQueueStepCB aResponseCB, auto... aParams){
      ApiQueuePtr<Tcomm, Tcb> thisPtr = getptr();

      auto singleStep = [=](){
        comm->apiRequest(
          command,
          [=](Tcb aResult, p44::ErrorPtr aError){
            thisPtr->errorStep = aResponseCB(aResult, aError);
            thisPtr->next_step();
          },
          aParams...);
      };

      stepsVec.push_back(singleStep);
    }

    void execute(ApiQueueCB aFinishCB)
    {
      finishCB = aFinishCB;
      if(stepsVec.empty()) return ;
      stepIter = stepsVec.begin();
      (*stepIter)();
    }

  };



} // namespace vdcutils