/*
 *  Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of vdc-utils
 */


#include "p44vdc/vdc_common/vdchost.hpp"

#if !DISABLE_DISCOVERY
#include "p44vdc/vdc_common/discovery.hpp"
#endif

#ifndef P44_JSON_API_SUPPORT
  #if defined(__APPLE__)
    #define P44_JSON_API_SUPPORT 1
  #else
    #define P44_JSON_API_SUPPORT 0
  #endif
#endif


// APIs to be used
#include "p44vdc/vdc_common/pbufvdcapi.hpp"
#include "p44vdc/vdc_common/device.hpp"
#include <boost/optional.hpp>

namespace p44 {

typedef std::list<VdcPtr> Vdcs;
typedef std::vector<CmdLineOptionDescriptor> Options;

/// Main program for dSS hosted vdc for specific 3rd party device support
class ApplicationVdc : public CmdLineApp
{
    typedef CmdLineApp inherited;

    MLTicket terminateTicket;

protected:
    // the device container
    // Note: must be a intrusive ptr, as it is referenced by intrusive ptrs later. Statically defining it leads to crashes.
    VdcHostPtr vdcHost;
    virtual Vdcs createVdcs() = 0;
    virtual Options vdcOptions() = 0;

  public:
    ApplicationVdc(VdcHostPtr aVdcHost);

    int main();
    void initialize();
    void initialized(ErrorPtr aError);
    void collectDevices(RescanMode aRescanMode);
    void devicesCollected(ErrorPtr aError);
    void parseArgs(int argc, char **argv);
    string getSslCertConfig();

    boost::optional<string> getStringOption(const string& aOptionName);
    bool getStringOption(const char *aOptionName, string &aString) { return inherited::getStringOption(aOptionName, aString); }

#if !DISABLE_DISCOVERY
    void initDiscovery();
#endif // !DISABLE_DISCOVERY

};

} // namespace p44
