/*
 *  Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of vdc-utils
 */


#pragma once

#include <functional>
#include <error.hpp>

namespace vdcutils {

template<typename... Args>
class Callback { };

template<typename... Args>
class Callback<void(Args...)>
{
    using functor_t = std::function<void(Args...)>;
    functor_t functor;

  public:

    template <typename Functor>
    Callback(Functor aFunction) :
    functor(std::move(aFunction)) {}

    Callback(const Callback& aCallback) = default;

    Callback():
      functor() {}

    template <typename... Params>
    void operator()(Params&&...args) const
    {
      if (!functor) {
        return;
      }
       functor(std::forward<Params>(args)...);
     }

    operator bool() const {
      return static_cast<bool>(functor);
    }

};

using ErrorCB = Callback<void(p44::ErrorPtr)>;

} // namespace vdcutils
