/*
 *  Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of vdc-utils
 */

// File scope debugging options
// - Set ALWAYS_DEBUG to 1 to enable DBGLOG output even in non-DEBUG builds of this file
#define ALWAYS_DEBUG 0
// - set FOCUSLOGLEVEL to non-zero log level (usually, 5,6, or 7==LOG_DEBUG) to get focus (extensive logging) for this file
//   Note: must be before including "logger.hpp" (or anything that includes "logger.hpp")
#define FOCUSLOGLEVEL 7

#include "client.hpp"
#include "httpcomm.hpp"

#if __cplusplus >= 201402L

using namespace p44;
using namespace vdcutils;

string vdcutils::toString(AccountStatus aValue)
{
  switch(aValue) {
  case AccountStatus::disconnected : return "disconnected";
  case AccountStatus::offline : return "offline";
  case AccountStatus::connected : return "connected";
  }
  return "unknown";
}

AccountStatus vdcutils::updateAccountStatus(ErrorPtr aError, IsAuthErrorCB aIsAuthError/*=isAuthorizationError*/)
{
  if (Error::isOK(aError)) {
    return AccountStatus::connected;
  }

  if (isHttpCommError(aError) || isServerError(aError)) {
    LOG(LOG_ERR, "Offline: %s", aError->description().c_str());
    return AccountStatus::offline;
  } else if (aIsAuthError(aError)) {
    LOG(LOG_ERR, "Authorization Error %s %d", aError->getErrorMessage(), aError->getErrorCode());
    return AccountStatus::disconnected;
  } else {
    LOG(LOG_ERR, "Communication Error %s %d", aError->getErrorMessage(), aError->getErrorCode());
    return AccountStatus::connected;
  }

}

bool vdcutils::isHttpCommError(p44::ErrorPtr aError)
{
  return !p44::Error::isOK(aError) &&
          aError->isDomain(HttpCommError::domain());
}

bool vdcutils::isServerError(p44::ErrorPtr aError)
{
  return aError &&
          aError->isDomain(p44::WebError::domain()) &&
            (aError->getErrorCode()/100 == 5); /* all 500s */
}

bool vdcutils::isAuthorizationError(p44::ErrorPtr aError)
{
  return aError &&
          (aError->getErrorCode() == HTTP_UNAUTHORIZED_ERR_CODE ||
           aError->getErrorCode() == HTTP_FORBIDDEN_ERR_CODE);
}



#endif

