/*
 *  Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of vdc-utils
 */
#pragma once

#if __cplusplus >= 201402L
#include "iconnectable.hpp"
#include <utility>
#include <type_traits>

namespace vdcutils {


template <typename Combiner>
struct ReturnType {
  using type = typename Combiner::result_type;
};

template <>
struct ReturnType<void> {
  using type = void;
};


template <typename Signature, typename Combiner>
struct SignalType {
  using type = typename boost::signals2::signal<Signature, Combiner>;
};

template <typename Signature>
struct SignalType<Signature, void> {
    using type = typename boost::signals2::signal<Signature>;
};


template <typename Signature, typename Combiner = void>
class Connectable : public IConnectable<Signature>
{
  using Signal_t = typename SignalType<Signature, Combiner>::type;
  using inherited = IConnectable<Signature>;
  using Return_t = typename ReturnType<Combiner>::type;

  Signal_t signal;

public:

  virtual ~Connectable() {}
  virtual boost::signals2::scoped_connection registerCallback(const typename inherited::CallbackType& aCallback) override { return signal.connect(aCallback); }

  template <typename... Args>
  Return_t callSignal(Args&&... aArgs) { return signal(std::forward<Args>(aArgs)...); }
};


} // namespace vdcutils

#endif

