/*
 *  Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of vdc-utils
 */

#pragma once

#include "p44vdc/vdc_common/p44vdc_common.hpp"

#ifndef P44_JSON_API_SUPPORT
  #if defined(__APPLE__)
    #define P44_JSON_API_SUPPORT 1
  #else
    #define P44_JSON_API_SUPPORT 0
  #endif
#endif

#if P44_JSON_API_SUPPORT
  #include "p44vdc/vdc_common/p44_vdcd_host.hpp"
#else
  #include "p44vdc/vdc_common/vdchost.hpp"
#endif

#include "p44vdc/vdc_common/device.hpp"

namespace p44 {

#ifndef DSVDC_VENDOR_NAME
static const char* DSVDC_VENDOR_NAME = "digitalSTROM AG";
#endif
#ifndef DSVDC_DESCRIPTION_TEMPLATE
static const char* DSVDC_DESCRIPTION_TEMPLATE = "%M @ %S";
#endif
#ifndef DSVDC_VDCMODELNAME_TEMPLATE
static const char* DSVDC_VDCMODELNAME_TEMPLATE = "dS %m";
#endif

/// specific vdc host class for predefining vendor, model etc.
class DsVdcHost : public
  #if P44_JSON_API_SUPPORT
  P44VdcHost
  #else
  VdcHost
  #endif
{
  #if P44_JSON_API_SUPPORT
  typedef P44VdcHost inherited;
  #else
  typedef VdcHost inherited;
  #endif
  typedef PersistentParams inheritedParams;

  int vdcCount;
  string vdcConfiguration;
  MLTicket terminateTicket;
protected:
  // persistence implementation
  virtual size_t numFieldDefs() P44_OVERRIDE;
  virtual const FieldDefinition *getFieldDef(size_t aIndex) P44_OVERRIDE;
  virtual void loadFromRow(sqlite3pp::query::iterator &aRow, int &aIndex, uint64_t *aCommonFlagsP) P44_OVERRIDE;
  virtual void bindToStatement(sqlite3pp::statement &aStatement, int &aIndex, const char *aParentIdentifier, uint64_t aCommonFlags) P44_OVERRIDE;
public:
  DsVdcHost();

  virtual string vendorName() P44_OVERRIDE { return DSVDC_VENDOR_NAME; };

  void addNewVdc();
  void deleteVdc(VdcPtr aVdcPtr);

  vector<int> readInstanceNumbers();
  void saveInstanceNumbers(const vector<int>& aInstanceNumbers);
};

typedef boost::intrusive_ptr<DsVdcHost> DsVdcHostPtr;

} /* namespace p44 */

