/*
 *  Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of vdc-utils
 */

#pragma once

#include <type_traits>
#include <unordered_map>

namespace vdcutils {

  template<typename T, typename = std::enable_if_t<std::is_enum<T>::value>>
  auto constexpr toUnderlyingType(T aVal)
  {
    return static_cast<std::underlying_type_t<T>>(aVal);
  }



  template<typename Enum_t, typename = std::enable_if_t<std::is_enum<Enum_t>::value>>
  class Enum;


  template<typename Enum_t>
  class Enum<Enum_t>
  {
    using NameMap_t = std::unordered_map<Enum_t, std::string>;
    const NameMap_t names;
    public:
    Enum(const NameMap_t& aNames):
    names(aNames)
    {

    }

    std::optional<std::string> toString(Enum_t aEnumValue) const
    {
      auto it = names.find(aEnumValue);

      if (it == names.end()) {
        return std::nullopt;
      }

      return it->second;
    }

    std::optional<Enum_t> fromString(const std::string& aEnumName) const
    {
      auto it = find_if(names.begin(), names.end(), [aEnumName](auto& aElement){ return aEnumName == aElement.second; });

      if (it == names.end()) {
        return std::nullopt;
      }

      return it->first;
    }
  };

} // namespace vdcutils

