/*
 *  Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of vdc-utils
 */

#pragma once
#include "p44vdc_common.hpp"

#if __cplusplus >= 201402L

#include "thirdparty/boost/request.hpp"
#include "jsoncomm.hpp"
#include "httpcomm.hpp"
#include "jsonwebclient.hpp"


namespace p44 {

struct Request {
    http::server::request head;
    string body;
};


class HttpServer : public SocketComm {

    using inherited = SocketComm;

    static const int MAX_CONNECTIONS = 10;
    static const MLMicroSeconds CONNECTION_TIMEOUT = 5*Second;
    static string HTTP_RESPONSE_OK;

    list<pair<JsonCommPtr, MLMicroSeconds>> connections;            ///< event channel connections with timestamp

    MLTicket timeoutRoutineTicket;
    HttpCommCB dataHandler;
    JsonWebClientCB jsonDataHandler;

  public:
    HttpServer(MainLoop &aMainLoop, const string& aHostname="localhost", const string& aPort="80");
    virtual ~HttpServer();
    ErrorPtr startServer();
    void setDataHandler(HttpCommCB aDataHandler) { dataHandler = aDataHandler; }
    void setJsonDataHandler(JsonWebClientCB aDataHandler) { jsonDataHandler = aDataHandler; }


  protected:
    /// Create event connection pointer
    SocketCommPtr configApiConnectionHandler(SocketCommPtr aServerSocketCommP);

    /// Request handler
    virtual void rawRequestHandler(JsonCommPtr aJsonComm, ErrorPtr aError, string& aRawHttpReq);

    /// Terminate connections with timeout
    void timeoutRoutine();

};
using HttpServerPtr = boost::intrusive_ptr<HttpServer>;




} // namespace p44

#endif // c++14
