/*
 *  Copyright (c) 2020 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of vdc-utils
 */

#include "jsonutils.hpp"

using namespace vdcutils;


p44::JsonObjectPtr json::get(p44::JsonObjectPtr aJson, const std::string& aName)
{
  if (!aJson) {
    throw JsonHelperException("JSON element " + aName + " not found. Empty JSON");
  }
  return get(*aJson, aName);
}


p44::JsonObjectPtr json::get(p44::JsonObject& aJson, const std::string& aName)
{
  auto elem = aJson.get(aName.c_str());
  if (!elem) {
    throw JsonHelperException("JSON element " + aName + " not found.");
  }
  return elem;
}


std::vector<std::string> json::getStringArray(p44::JsonObjectPtr aArrayJson)
{
  if (!aArrayJson) {
    throw JsonHelperException("JSON array object is empty.");
  }
  return getStringArray(*aArrayJson);
}


std::vector<std::string> json::getStringArray(p44::JsonObject& aArrayJson)
{
  std::vector<std::string> ret;
  if (!aArrayJson.isType(json_type_array)) {
    throw JsonHelperException("JSON is not an array");
  }

  for (int i=0; i<aArrayJson.arrayLength(); ++i) {
    auto elem = aArrayJson.arrayGet(i);
    if (!elem) {
      throw JsonHelperException("Array element is an empty JSON");
    }
    ret.emplace_back(elem->stringValue());
  }

  return ret;
}


std::vector<std::string> json::getStringArray(p44::JsonObjectPtr aJson, const std::string& aName)
{
  if (!aJson) {
    throw JsonHelperException("JSON array " + aName + " not found.Empty JSON");
  }
  return getStringArray(*aJson, aName);
}


std::vector<std::string> json::getStringArray(p44::JsonObject& aJson, const std::string& aName)
{
  auto arrayJson = get(aJson, aName);
  return getStringArray(arrayJson);
}


std::optional<std::string> json::getJsonStringValue(p44::JsonObjectPtr aRootJson, const std::string& aKey)
{
  if (aRootJson) {
    auto keyJsonObject = aRootJson->get(aKey.c_str());
    if (keyJsonObject) return keyJsonObject->stringValue();
  }
  return std::nullopt;
}


std::optional<int> json::getJsonInt32Value(p44::JsonObjectPtr aRootJson, const std::string& aKey)
{
  if (aRootJson) {
    auto keyJsonObject = aRootJson->get(aKey.c_str());
    if (keyJsonObject) return keyJsonObject->int32Value();
  }
  return std::nullopt;
}


std::optional<double> json::getJsonDoubleValue(p44::JsonObjectPtr aRootJson, const std::string& aKey)
{
  if (aRootJson) {
    auto keyJsonObject = aRootJson->get(aKey.c_str());
    if (keyJsonObject) return keyJsonObject->doubleValue();
  }
  return std::nullopt;
}


std::optional<bool> json::getJsonBoolValue(p44::JsonObjectPtr aRootJson, const std::string& aKey)
{
  if (aRootJson) {
    auto keyJsonObject = aRootJson->get(aKey.c_str());
    if (keyJsonObject) return keyJsonObject->boolValue();
  }
  return std::nullopt;
}


p44::JsonObjectPtr json::newJsonItem(bool aValue)
{
  return p44::JsonObject::newBool(aValue);
}


p44::JsonObjectPtr json::newJsonItem(const std::string& aValue)
{
  return p44::JsonObject::newString(aValue);
}
