/*
 *  Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of vdc-utils
 */
#pragma once

#if __cplusplus >= 201103L // __func__ is from C99

#include "p44utils/logger.hpp"

#ifdef __PRETTY_FUNCTION__
  #define LOG_SHORT(logger, lvl) logger(lvl, __PRETTY_FUNCTION__)
  #define LOG_FUNCTION(logger, lvl, fmt, ...) logger(lvl, __PRETTY_FUNCTION__ " : " fmt, ##__VA_ARGS__)
#else
  #define LOG_F_FILENAME (strrchr(__FILE__, '/') ? strrchr(__FILE__, '/') + 1 : __FILE__)
  #define LOG_S_FORMAT "%s : %s"
  #define LOG_F_FORMAT(fmt) LOG_S_FORMAT " : " fmt
  #define LOG_M_FORMAT(module, fmt) "[" module "] " fmt
  #define LOG_F_FUNCTION __func__

  #define LOG_SHORT(logger, lvl) logger(lvl, LOG_S_FORMAT, LOG_F_FILENAME, LOG_F_FUNCTION)
  #define LOG_MODULE(logger, lvl, module,  fmt, ...) logger(lvl, LOG_M_FORMAT(module, fmt), ##__VA_ARGS__)
  #define LOG_FUNCTION(logger, lvl, fmt, ...) logger(lvl, LOG_F_FORMAT(fmt), LOG_F_FILENAME, LOG_F_FUNCTION, ##__VA_ARGS__)
#endif

#define LOG_S(lvl) LOG_SHORT(LOG, lvl)
#define LOG_F(lvl, ...) LOG_FUNCTION(LOG, lvl, ##__VA_ARGS__)
#define LOG_M(lvl, module, ...) LOG_MODULE(LOG, lvl, module, ##__VA_ARGS__)

#define ALOG_S(lvl) LOG_SHORT(ALOG, lvl)
#define ALOG_F(lvl, ...) LOG_FUNCTION(ALOG, lvl, ##__VA_ARGS__)

#else

#define LOG_S(lvl)
#define LOG_F(lvl, fmt, ...) LOG(lvl, fmt, ##__VA_ARGS__)

#define ALOG_S(lvl)
#define ALOG_F(lvl, fmt, ...) LOG(lvl, fmt, ##__VA_ARGS__)

#endif
