/*
 *  Copyright (c) 2019 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of vdc-utils
 */

#include "macaddressutils.hpp"
#include "p44utils/logger.hpp"
#include "p44utils/macaddress.hpp"
#include "p44utils/utils.hpp"
#include <cstdlib>
#include <iomanip>
#include <sstream>
#include <algorithm>


using namespace vdcutils;


bool vdcutils::operator==(const MacAddress& aMacAddressL, const MacAddress& aMacAddressR) {
  return std::equal(aMacAddressL.toBytes().begin(), aMacAddressL.toBytes().end(), aMacAddressR.toBytes().begin());
}


MacAddress::MacAddress(const MacAddressBytes& aBinaryMac):
  mMacAddress(aBinaryMac)
{

}


MacAddress MacAddress::fromInteger(uint64_t aMacInteger)
{
  return MacAddress {
    {
      static_cast<std::byte>((aMacInteger >> 40) & 0xFF),
      static_cast<std::byte>((aMacInteger >> 32) & 0xFF),
      static_cast<std::byte>((aMacInteger >> 24) & 0xFF),
      static_cast<std::byte>((aMacInteger >> 16) & 0xFF),
      static_cast<std::byte>((aMacInteger >> 8) & 0xFF),
      static_cast<std::byte>((aMacInteger) & 0xFF)
    }
  };
}


std::optional<MacAddress> MacAddress::fromIp(const std::string& aIpString)
{
  uint64_t macAddress;
  if (!p44::getMacAddressByIpv4(p44::stringToIpv4(aIpString.c_str()), macAddress)) {
    LOG(LOG_DEBUG, "Cannot get device mac address (IP : %s)", aIpString.c_str());
    return std::nullopt;
  }
  LOG(LOG_NOTICE, "MAC Address: '%lx'", macAddress);
  return fromInteger(macAddress);
}


std::optional<MacAddress> MacAddress::fromString(std::string aMacString)
{
  aMacString.erase(std::remove(aMacString.begin(), aMacString.end(), ':'), aMacString.end());
  uint64_t macAddress = std::strtoull(aMacString.c_str(), nullptr, 16);
  if (macAddress == 0){
    return std::nullopt;
  }
  return fromInteger(macAddress);
}


std::string MacAddress::toString() const
{
  std::stringstream sstr;
  for (auto byte : mMacAddress) {
    sstr << std::setw(2) << std::setfill('0') << std::hex << std::to_integer<uint>(byte);
  }
  return sstr.str();
}


