/*
 *  Copyright (c) 2019 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of vdc-utils
 */


#include "messagebox.hpp"
#include <optional>
#include "enumutils.hpp"


using namespace vdcutils;


namespace {

  std::optional<std::string> toString(MessageLevel aLevel)
  {
    static const Enum<MessageLevel> msgLevels {
      {
        { MessageLevel::Success,  "success" },
        { MessageLevel::Info,     "info"    },
        { MessageLevel::Warning,  "warning" },
        { MessageLevel::Error,    "error"   }
      }
    };

    return msgLevels.toString(aLevel);
  }

}


p44::JsonObjectPtr MessageBox::Message::toJson()
{
  auto json = p44::JsonObject::newObj();
  auto jTimestamp = p44::JsonObject::newInt64(timestamp);
  json->add("timestamp", jTimestamp);

  auto levelStr = toString(level);
  if (!levelStr) {
    LOG(LOG_ERR, "Invalid message level");
    return nullptr;
  }

  auto jLevel = p44::JsonObject::newString(*levelStr);
  json->add("level", jLevel);

  json->add("message", message);

  return json;
}

MessageBox::MessageBox(size_t aSize) :
  mMessages(Messages_t(aSize))
{

}


void MessageBox::addMessage(MessageLevel aLevel, p44::JsonObjectPtr aMessage)
{
  mMessages.push_back(
      Message {
        .timestamp = std::time(nullptr),
        .level = aLevel,
        .message = aMessage
      }
  );
}


p44::JsonObjectPtr MessageBox::toJson()
{
  auto json = p44::JsonObject::newObj();
  auto jMessages = p44::JsonObject::newArray();

  for (auto& msg : mMessages) {
    if (auto jMsg = msg.toJson()) {
      jMessages->arrayAppend(jMsg);
    }
  }

  json->add("messages", jMessages);
  return json;
}

