/*
 *  Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of vdc-utils
 */
#pragma once

#if __cplusplus >= 201402L
#include <utility>
#include <string>
#include <memory>
#include <list>
#include <functional>
#include "ipersistentstoragewithrowid.hpp"

namespace vdcutils {

template <typename T>
struct PersistentRecordRef {
  std::string name;
  T& value;
};

template <typename T>
struct PersistentRecordCB {
  using LoadCB_t = std::function<void(const T&)>;
  using SaveCB_t = std::function<T()>;
  std::string name;
  LoadCB_t loadCB;
  SaveCB_t saveCB;
};

using StringList = std::list<std::string>;

class IPersistentStorageFactory
{
public:

  virtual ~IPersistentStorageFactory() {}

  virtual void addRecord(const PersistentRecordRef<std::string>& aRecord) = 0;
  virtual void addRecord(const PersistentRecordCB<std::string>& aRecord) = 0;
  virtual void addRecord(const PersistentRecordRef<bool>& aRecord) = 0;
  virtual void addRecord(const PersistentRecordCB<bool>& aRecord) = 0;
  virtual void addRecord(const PersistentRecordRef<StringList>& aRecord) = 0;
  virtual void addRecord(const PersistentRecordCB<StringList>& aRecord) = 0;

  virtual std::unique_ptr<IPersistentStorageWithRowId> createStorageWithRowId(const std::string& aRowId, const std::string& aTableName) = 0;

};

} // namespace vdcutils

#endif

