/*
 *  Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of vdc-utils
 */
#pragma once

#if __cplusplus >= 201402L

#include "ipersistentstoragefactory.hpp"
#include "persistentstorage.hpp"


namespace vdcutils {


class PersistentStorageFactory : public IPersistentStorageFactory
{
public:
  PersistentStorageFactory(p44::ParamStore& aParamStore) : paramStore(aParamStore) {}

  void addRecord(const PersistentRecordRef<std::string>& aRecord) override
  {
    stringRecordsRef.push_back(aRecord);
  }

  void addRecord(const PersistentRecordCB<std::string>& aRecord) override
  {
    stringRecordsCB.push_back(aRecord);
  }

  void addRecord(const PersistentRecordRef<bool>& aRecord) override
  {
    boolRecordsRef.push_back(aRecord);
  }

  void addRecord(const PersistentRecordCB<bool>& aRecord) override
  {
    boolRecordsCB.push_back(aRecord);
  }

  void addRecord(const PersistentRecordRef<std::list<std::string>>& aRecord) override
  {
    stringListRecordsRef.push_back(aRecord);
  }

  void addRecord(const PersistentRecordCB<std::list<std::string>>& aRecord) override
  {
    stringListRecordsCB.push_back(aRecord);
  }

  std::unique_ptr<IPersistentStorageWithRowId> createStorageWithRowId(const std::string& aRowId, const std::string& aTableName) override
  {
    auto ret = std::make_unique<
      PersistentStorageWithRowId<p44::PersistentParams>>(
          aRowId,
          aTableName,
          std::make_tuple(
            stringRecordsRef,
            stringRecordsCB,
            boolRecordsRef,
            boolRecordsCB,
            stringListRecordsRef,
            stringListRecordsCB),
          paramStore);

    stringRecordsRef.clear();
    stringRecordsCB.clear();
    boolRecordsRef.clear();
    boolRecordsCB.clear();
    stringListRecordsRef.clear();
    stringListRecordsCB.clear();
    return ret;
  }

private:
  p44::ParamStore& paramStore;
  std::vector<PersistentRecordRef<std::string>> stringRecordsRef;
  std::vector<PersistentRecordCB<std::string>> stringRecordsCB;
  std::vector<PersistentRecordRef<bool>> boolRecordsRef;
  std::vector<PersistentRecordCB<bool>> boolRecordsCB;
  std::vector<PersistentRecordRef<std::list<std::string>>> stringListRecordsRef;
  std::vector<PersistentRecordCB<std::list<std::string>>> stringListRecordsCB;
};

} // namespace vdcutils

#endif

