/*
 *  Copyright (c) 2020 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of vdc-utils
 */
#pragma once

#include "p44utils/ssdpsearch.hpp"

namespace vdcutils {

class ISsdpDiscovery
{
  public:
  virtual ~ISsdpDiscovery()=default;
  virtual void stopSearch() = 0;
  virtual void startSearchForTarget(
    p44::SsdpSearchCB aSearchResultHandler,
    const std::string& aSearchTarget) = 0;
};


class SsdpDiscovery : public ISsdpDiscovery
{
  p44::SsdpSearchPtr mSsdpSearch;

  public:
  SsdpDiscovery(p44::MainLoop& aMainLoop) :
    mSsdpSearch(vdcutils::make_intrusive<p44::SsdpSearch>(aMainLoop))
  {

  }
  private:
  void stopSearch() { mSsdpSearch->stopSearch(); }
  void startSearchForTarget(
    p44::SsdpSearchCB aSearchResultHandler,
    const std::string& aSearchTarget) {
      mSsdpSearch->startSearchForTarget(aSearchResultHandler, aSearchTarget.c_str(), false, true);
  }
};

} // namespace vdcutils
