# VDC Unit tests #

Unit test templates and utils dedicated for p44vdc based vDC

### Overview ###

Unit test templates and utilities lets you create quick unit test,
which will check basic initialization of vDC device and its configuration.

### Features ###

 * Provides mocks for basic system components
 * Provides test functions to run the same test among different environment scenarios
 * Tests vDC device configuration and capabilities(actions, sensors, events, etc) defined in json file

### Requirements ###

 * google test
 * google mock
 * p44vdc
 * p44utils


## Getting Started wit vDC device test ##

 1. Create device test fixture inheriting from `DeviceTest`
 2. Create device in `ctor` or `SetUp` method, store it in class member `DevicePtr device`
 3. Provide your custom class extentions
 4. Provide json test configuration in [test.json](#test.json) file.
 5. Create json object `jsonTest` in ctor.
 6. Use test [Test Functions](#Test-Functions) in `TEST_F`
 7. Run test.


# test.json

This file defines expected device configuration.
`test.json` file is required in root location.
Json schema with examples:
```
{
  "HoodDevice":{
    "events": {
      "expected": ["ProgramFinished", "LocallyOperated", "ProgramStarted"],
      "unexpected": [ "ProgramAborted",  "AlarmClockElapsed"]
    },
    "actions": {
      "expected":
      [
        {"name": "PowerOff"},
        {
          "name": "AutoRunFan",
          "params":
          [
            {
              "name": "TemperatureTrigger",
              "description": {
                "default":50.000000,
                "max":100.000000,
                "min":1.000000,
                "optional":true,
                "resolution":1.000000,
                "siunit":"celsius",
                "symbol":"°C",
                "type":"numeric"
                }
            },
            {
              "name": "HumidityTrigger"
            }
          ]
        }
      ],
      "unexpected": ["ActFanLevel4", "ActFanIntense3"]
    },
    "properties": {
      "expected":
      [
        {
          "name": "ProgramName",
          "description": {
            "type":"string"
          }
        },
        {
          "name": "RemainingProgramTime",
          "description": {
            "type":"numeric",
            "siunit":"second",
            "symbol":"S",
            "min":0.000000,
            "max":86340.000000,
            "resolution":1.000000
          }
        }
      ],
      "unexpected":
      [
        "unexpectedPropertyName"
      ]
    },
    "sensors": {
      "expected":
      [
        {
          "name": "sensortemperature",
          "description": {
            "aliveSignInterval":0.000000,
            "dsIndex":0,
            "max":100.000000,
            "min":1.000000,
            "name":"Current temperature",
            "resolution":1.000000,
            "sensorType":1,
            "sensorUsage":4,
            "siunit":"celsius",
            "symbol":"°C",
            "type":"sensor",
            "updateInterval":0.000000,
            "x-p44-behaviourType":"sensor"
          }
        },
        { "name": "testsensor"}
      ],
      "unexpected":
      [
        "unexpectedSensorName"
      ]
    },
    "states": {
      "expected":
      [
        {
          "name": "OperationMode",
          "expected": ["ModeInactive", "ModeRun"],
          "unexpected": ["ModeReady", "ModeDelayedStart"]
        },
        {
          "name": "RemoteControl",
          "expected": ["RemoteControlActive", "RemoteControlInactive", "RemoteStartActive"],
          "unexpected": ["RemoteControlErrorTC"]
        }
      ],
      "unexpected": ["PowerState"]
    },
    "scenes": {
      "FIRE": "Stop"
    },
    "identify": {
      "gtin": "gs1:(01)7640156794298",
      "dSUID": "D2560650D24051BAC04CDA5BD1AD5EEA00",
      "hwGUID": "123456",
      "hwModelGUID": "123456",
      "name": "Bosch Hood XYZ",
      "modelName": "XYZ",
      "vendorName": "Bosch"
    },
    "outputs": {
      "expected": [
        {
          "name": "videoInputSource",
          "description": {
            "channelType": 26,
            "max": 17.000000,
            "min": 0.000000,
            "name": "video input source",
            "resolution": 1.000000,
            "siunit": "none",
            "symbol": ""
          },
          "values": {
            "TV": 0,
            "Analog_TV": 1,
            "DVB_T": 2,
            "DVB_S": 3,
            "Freesat": 4,
            "DVB_C": 5,
            "SAT_IP": 6,
            "Video_1": 7,
            "Video_2": 8,
            "Component_1": 9,
            "HDMI_1": 10,
            "HDMI_2": 11,
            "HDMI_3": 12,
            "HDMI_4": 13,
            "Display_Port": 14,
            "SD_USB": 15,
            "Apps": 16,
            "My_Apps": 17
          }
        },
        {
          "name": "videoStation",
          "description": {
            "channelType": 25,
            "max": 9999.000000,
            "min": 0.000000,
            "name": "video station",
            "resolution": 1.000000,
            "siunit": "none",
            "symbol": ""
          }
        }
      ],
      "unexpected": []
    }
  }
}
```
### testJson schema ###

  * `events`, json schema:
    * `expected` - json string array - defines entries expected in device
    * `unexpected` - json string array - optional, defines entries which cannot be added to the device

  * `actions` json schema:
    * `expected` - json object array - defines actions expected in device and its params
      * `name` - json string - action name
      * `params`- json object array - optional, defines expected action parameters
        * `name` - json string - action parameter name
        * `description` - json object - optional, defines parameter description
          * `default` - json value - optional, default parameter value
          * `max` - json value - optional, max parameter value
          * `min` - json value - optional, min parameter value
          * `optional` - json bollean - optional, defines if parameter is optional
          * `siunit` - json string - optional, si-unit name
          * `symbol` - json string - optional, si-unit symbol
          * `type` - json string - optional, type of parameter
    * `unexpected` - json string array - optional, defines entries which cannot be added to the device

  * `properties` json schema:
    * `expected` - json object array - defines properties expected in device and descriptions
      * `name` - json string - property name
      * `description`- json object array - optional, defines expected action parameters
        * `type` - json string - optional, type of property
        * `siunit` - json string - optional, si-unit name
        * `symbol` - json string - optional, si-unit symbol
        * `max` - json value - optional, max value
        * `min` - json value - optional, min value
        * `resolution` - json value - optional, resolution of value
    * `unexpected` - json string array - optional, defines entries which cannot be added to the device

  * `sensors` json schema:
    * `expected` - json object array - defines sensors expected in device and its descriptions
      * `name` - json string - sensor name
      * `description`- json object array - optional, defines expected action parameters
        * `aliveSignInterval` - json value - optional, sensor aliveSIgnInterval
        * `dsIndex` - json int value - optional, sensor index
        * `max` - json value - optional, max sensor value
        * `min` - json value - optional, min sensor value
        * `name` - json string value - optional, sensor name (description)
        * `resolution` - json value - optional, sensor value resolution
        * `sensorType` - json int value - optional, sensor type
        * `sensorUsage` - json int value - optional, sensor usage
        * `siunit` - json string value - optional, si-unit name
        * `symbol` - json string value - optional, si-unit symbol
        * `type` - json string value - optional, ds element type (probably constatnt "sensor")
        * `updateInterval` - json value - optional, sensor update interval
    * `unexpected` - json string array - optional, defines entries which cannot be added to the device

  * `states` json schema:
    * `expected` - json object array - defines expected device states and values
      * `name` - json string - device state name
      * `expected` - json string array - defines expected enum values
      * `unexpected`- json string array - optional, defines unexpected enum values
    * `unexpected` - json string array - optional, defines unforseen device states

  * `identify` - json object - device identification data
    * `gtin` - json string - optional, device gtin
    * `dSUID` - json string - optional, device derived dsuid
    * `hwGUID` - json string - optional, device hardware unique identifier
    * `hwModelGUID` - json string - optional, device model unique identifier
    * `name` - json string - optional, device name
    * `modelName` - json string - optional, device model name
    * `vendorName` - json string - optional, device vendor name
  * `outputs` json schema:
    * `expected` - json object array - defines outputs expected in device and its descriptions
      * `name` - json string value - optional, sensor name (description)
      * `description`- json object array - optional, defines expected action parameters
        * `max` - json value - optional, max sensor value
        * `min` - json value - optional, min sensor value
        * `resolution` - json value - optional, sensor value resolution
        * `channel` - json int value - optional, sensor type
        * `siunit` - json string value - optional, si-unit name
        * `symbol` - json string value - optional, si-unit symbol
      * `values` - json list - optional, enum values descriptor
    * `unexpected` - json string array - optional, defines entries which cannot be added to the device


# Test Functions

Check descriptions. Test looks for expected entries and not expected ones, checks entries quantity:
  * testEvents() - test device events availability
  * testActions() - test device actions and parameters
  * testProperties() - test device properties availability
  * testStates() - test device states and state enum values
  * testScenes() - test commands set for scene
  * testSensors() - test device sensors and its description
  * testIdentifyData() - test identification data
  * testOutput() - test output channels

## Persistent Storage Test
Check custom fields defined in persistent params storage.

## VdcStorageTest class
Test custom fields defined in vDC class.

### Test Functions
 * `void testFieldDefinitions(ExpectedFieldDefs aExpectedFields)` - test expected field definitions, its name and data type

### Example
```
namespace p44 {
using namespace ::testing;

class VdcStorageTest : public VdcStorageTest<p44::testdevice::Vdc>
{
  public:
  using inherited = VdcStorageTest<p44::testdevice::Vdc>;

  VdcStorageTest() :
  inherited()
  { }
};

TEST_F(VdcStorageTest, vdcStorageTest)
{
  testFieldDefinitions({{"testVdcField1", SQLITE3_TEXT}});
}

} // namespace p44
```

## StorageTest class
Test fields created in custom class.

### Test Functions
 * `void testStorageFactory(const std::string &aExpRowId, const std::string &aExpTableName)` - test expected row id and table name
 * `void testCreateStringField(ExpectedStringFields &aExpectedFields)` - test expected string fields and default values

### Example
```
namespace p44 {
using namespace ::testing;

class TestDStorageTest : public StorageTest
{
public:
    VdcHost host;
    StrictMock<VdcMock> vdc;
    DevicePtr device;

    TestDStorageTest() :
    vdc(0, &host, 0)
    { }
};

TEST_F(TestDStorageTest, storageTest)
{
    ExpectedStringFields expectedStringRecords = {
        std::make_pair("testField1", ""),
        std::make_pair("testField2", "defaultStringValue")
    };

    testStorageFactory("MyRowId", "MyTableName"); // enqueue EXPECTED_CALLs
    testCreateStringField(expectedStringRecords); // enqueue EXPECTED_CALLs
    device = make_intrusive<testdevice::Device>(&vdc, storageFactoryMock); // calls enqueued methods
}

} // namespace p44
```

