/*
 *  Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of vdc-utils
 */

#pragma once

#include "gtest/gtest.h"
#include "mocks/vdcmock.hpp"

#include "singledevice.hpp"
#include "jsonvdcapi.hpp"
#include "vdc-utils/makeintrusive.hpp"
#include "boost/optional.hpp"

namespace vdcunittest {


class DeviceTest : public testing::Test
{
  protected:

    p44::VdcHost host;
    testing::StrictMock<VdcMock> vdc;
    p44::DevicePtr device;

    p44::JsonObjectPtr jsonTest;

    MOCK_METHOD1(statusCbMock, void(p44::ErrorPtr));

    DeviceTest();
    void identifyDeviceWithCallback();
    void identifyDeviceWithReturn();
    p44::ErrorPtr handleMethod(const string& aMethod, p44::ApiValuePtr aParams);
    using ContainerTest = function<void(p44::ApiValuePtr)>;
    void getContainer(const std::string& aContainer, ContainerTest aCb);
    void testContainer(const std::string& aContainerName, const std::string& aTestJsonKey);
    void testContainerWithDescription(const std::string& aContainerName, const std::string& aTestJsonKey);
    void compareKeyValueContainers(p44::ApiValuePtr aDeviceContainer, p44::JsonObjectPtr aTestJson);

    static p44::ApiValuePtr getDeviceStateEnumValues(p44::ApiValuePtr aContainer, const std::string& aDeviceState);
    p44::ApiValuePtr getDeviceOutputValues(p44::ApiValuePtr aContainer, const std::string &aDeviceState);
    static size_t getContainerSize(p44::ApiValuePtr aContainer);
    static bool isContainerEmpty(p44::ApiValuePtr aContainer);
    static void testExpectedValues(p44::ApiValuePtr aContainer, p44::JsonObjectPtr aValues);
    static void testUnexpectedValues(p44::ApiValuePtr aContainer, p44::JsonObjectPtr aValues);
    static boost::optional<DsSceneNumber> sceneNumberFromName(const string& aSceneName);
    static void testActionParams(p44::ApiValuePtr aCurrentParams, p44::JsonObjectPtr aExpectedParams);
    static void testDescription(p44::ApiValuePtr aCurrentParamDesc, p44::JsonObjectPtr aTestValue);
    static p44::JsonObjectPtr getExpectedValues(p44::JsonObjectPtr aTestValues, size_t aExpectedSize);
    static std::string getName(p44::JsonObjectPtr aTestValue);
    static p44::ApiValuePtr getDeviceValue(p44::ApiValuePtr aContainer, const std::string& aValueName);

    // tests
    void testEvents();
    void testActions();
    void testProperties();
    void testStates();
    void testScenes();
    void testIdentificationData();
    void testSensors();
    void testOutputs();

};

} //namespace vdcunittest


