/*
 *  Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of vdc-utils
 */

#include "gtest/gtest.h"
#include "persistentstoragetest.hpp"


using namespace vdcunittest;
using namespace ::testing;
using namespace vdcutils;

void StorageTest::testStorageFactory(const std::string &aExpRowId, const std::string &aExpTableName)
{
  auto storage = std::make_unique<StrictMock<PersistentStorageWithRowIdMock>>();
  storageMock = storage.release();
  LOG(LOG_NOTICE, "Expecting Storage with row id '%s' and table name '%s'", aExpRowId.c_str(), aExpTableName.c_str());
  EXPECT_CALL(storageFactoryMock, createStorageWithRowIdProxy(aExpRowId, aExpTableName)).WillOnce(Return(storageMock));
  EXPECT_CALL(*storageMock, load());
}

void StorageTest::testCreateStringField(ExpectedStringFields &aExpectedFields)
{
  EXPECT_CALL(storageFactoryMock, addRecord(Matcher<const PersistentRecordRef<std::string> &>(_))).Times(aExpectedFields.size())
    .WillRepeatedly(Invoke([aExpectedFields](const PersistentRecordRef<std::string> &aValue) {
      LOG(LOG_NOTICE, "Looking for field '%s' with value '%s' in expected fields", aValue.name, aValue.value.c_str());
      auto found = any_of(aExpectedFields.begin(), aExpectedFields.end(), [=](auto aExpectedField) {
        return aExpectedField.first == aValue.name;
      });
      EXPECT_TRUE(found && !aExpectedFields.empty());
    }));
}

void StorageTest::testCreateStringListField(std::list<std::string> &aExpectedFields)
{
  EXPECT_CALL(storageFactoryMock, addRecord(Matcher<const PersistentRecordRef<std::list<std::string>> &>(_))).Times(aExpectedFields.size())
    .WillRepeatedly(Invoke([aExpectedFields](const PersistentRecordRef<std::list<std::string>>& aValue) {
        LOG(LOG_NOTICE, "Looking for field '%s' in expected fields", aValue.name);
        auto found = any_of(aExpectedFields.begin(), aExpectedFields.end(), [=](auto aExpectedField) {
          return aExpectedField == aValue.name;
        });
        EXPECT_TRUE(found && !aExpectedFields.empty());


    }));
}


