/*
 *  Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of vdc-utils
 */
#pragma once

#if __cplusplus >= 201402L

#include "p44vdc/vdc_common/singledevice.hpp"
#include "typedenumvaluedescriptor.hpp"
#include <exception>

namespace vdcutils {

template <typename T>
class TypedDeviceState : public p44::DeviceState
{
  using inherited = DeviceState;

  TypedEnumValueDescriptorPtr<T> desc;
public:

  TypedDeviceState(p44::SingleDevice &aSingleDevice,
                   const string& aStateId,
                   const string& aDescription,
                   TypedEnumValueDescriptorPtr<T> aStateDescriptor,
                   p44::DeviceStateWillPushCB aWillPushHandler) :
    inherited(aSingleDevice, aStateId, aDescription, aStateDescriptor, std::move(aWillPushHandler)),
    desc(aStateDescriptor)
  {
    if(desc == nullptr){
      throw std::invalid_argument("TypedDeviceState : aStateDescriptor == nullptr");
    }
  }

  TypedEnumValueDescriptorPtr<T> typedValue()
  {
    return desc;
  }

  virtual bool setAndPushValue(T aValue)
  {
    if (desc->setValue(aValue)) {
      return push();
    }
    return false;
  }

  bool set(T aValue)
  {
    return desc->setValue(aValue);
  }

  T get()
  {
    return desc->getValue();
  }

  T getPrevious()
  {
    return desc->getPreviousValue();
  }

  bool invalidateAndPush()
  {
    if (desc->invalidate()) {
      return push();
    }
    return false;
  }
};


template <typename T>
using TypedDeviceStatePtr = boost::intrusive_ptr<TypedDeviceState<T>>;

} // namespace vdcutils

#endif

