/*
 *  Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of vdc-utils
 */

// File scope debugging options
// - Set ALWAYS_DEBUG to 1 to enable DBGLOG output even in non-DEBUG builds of this file
#define ALWAYS_DEBUG 0
// - set FOCUSLOGLEVEL to non-zero log level (usually, 5,6, or 7==LOG_DEBUG) to get focus (extensive logging) for this file
//   Note: must be before including "logger.hpp" (or anything that includes "logger.hpp")
#define FOCUSLOGLEVEL 7


#include "udpserver.hpp"
#include "loggerext.hpp"

#if __cplusplus >= 201402L


using namespace p44;
using namespace vdcutils;


UdpServer::UdpServer(MainLoop &aMainLoop, const string& aHost, const string& aPort) :
  inherited(aMainLoop),
  host(aHost),
  port(aPort)
{
  setReceiveHandler([&](auto aError){ this->readData(aError); });
}


UdpServer::~UdpServer()
{
  try {
    closeConnection();
  }
  catch (const boost::bad_function_call& e) {
    LOG_F(LOG_ERR, "Bad function call exception thrown when closing connection");
  }

}


ErrorPtr UdpServer::startServer()
{
  closeConnection();
  setConnectionParams(host.c_str(), port.c_str(), SOCK_DGRAM, AF_INET);
  enableBroadcast(true);
  setConnectionStatusHandler([=](SocketCommPtr aSocketComm, ErrorPtr aError){ /*TODO: handle connection status*/ });
  return initiateConnection();
}


void UdpServer::readData(ErrorPtr aError)
{
  if (Error::isOK(aError)){

    auto dataSize = numBytesReady();
    vector<uint8_t> dataReceived(dataSize);
    ErrorPtr err;
    receiveBytes(dataSize, dataReceived.data(), err);

    if (rawDataHandler) {
      rawDataHandler(dataReceived, err);
    }

  }
}


#endif // c++14

