/*
 *  Copyright (c) 2019 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of vdc-utils
 */


#include "uinotificationmsg.hpp"


using namespace vdcutils;


UiNotificationBuilder& UiNotificationBuilder::operator<<(const UiNotificationTranslatable& aTranslatable)
{
  addParameter(content.tVars, aTranslatable);
  return *this;
}


UiNotificationBuilder& UiNotificationBuilder::operator<<(const UiNotificationUntranslatable& aUntranslatable)
{
  addParameter(content.uVars, aUntranslatable);
  return *this;
}


void UiNotificationBuilder::addParameter(UiNotificationBuilder::Vars& aVars, const UiNotificationParameter& aParameter)
{
  aVars.push_back(aParameter);
  content.txt << "${" << aParameter.key << "}";
}


p44::JsonObjectPtr UiNotificationBuilder::msg(const std::string& aId)
{
  auto json = p44::JsonObject::newObj();

  auto jId = p44::JsonObject::newString(aId);
  json->add("id", jId);
  auto jTxt = p44::JsonObject::newString(content.txt.str());
  json->add("txt", jTxt);
  if (auto jTr = translatableJson()) {
    json->add("translatables", jTr);
  }
  if (auto jUntr = untranslatableJson()) {
    json->add("untranslatables", jUntr);
  }

  flush();
  return json;
}


p44::JsonObjectPtr UiNotificationBuilder::translatableJson()
{
  return varsJson(content.tVars);
}


p44::JsonObjectPtr UiNotificationBuilder::untranslatableJson()
{
  return varsJson(content.uVars);
}


p44::JsonObjectPtr UiNotificationBuilder::varsJson(const Vars& aVars)
{
  if (aVars.empty()) {
    return nullptr;
  }
  auto json = p44::JsonObject::newObj();
  for (auto& var : aVars) {
    if (var.value.empty()) {
      throw UiNotificationBuilderException("Empty variable value");
    }
    auto jVal = p44::JsonObject::newString(var.value.c_str());
    json->add(var.key.c_str(), jVal);
  }
  return json;
}


void UiNotificationBuilder::flush()
{
  content = {};
}
