/*
 *  Copyright (c) 2019 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of vdc-utils
 */


#include <unordered_map>
#include <string>
#include <sstream>
#include "exception.hpp"
#include "jsonobject.hpp"
#include "logger.hpp"


namespace vdcutils {


class UiNotificationBuilderException : public vdcutils::Exception
{
  using vdcutils::Exception::Exception;
};



struct UiNotificationParameter
{
  std::string key;
  std::string value;
};


struct UiNotificationTranslatable : public UiNotificationParameter
{};


struct UiNotificationUntranslatable : public UiNotificationParameter
{};


class UiNotificationBuilder
{

  using Vars = std::list<UiNotificationParameter>;

  struct Content {
    std::stringstream txt = std::stringstream();
    Vars tVars;
    Vars uVars;
  } content;

  public:
  UiNotificationBuilder()
  {}

  UiNotificationBuilder& operator<<(const UiNotificationTranslatable& aTranslatable);
  UiNotificationBuilder& operator<<(const UiNotificationUntranslatable& aTranslatable);
  p44::JsonObjectPtr msg(const std::string& aId);

  template<typename Var>
  UiNotificationBuilder& operator<<(const Var& aVal)
  {
    content.txt << aVal;
    return *this;
  }

  private:
  p44::JsonObjectPtr translatableJson();
  p44::JsonObjectPtr untranslatableJson();
  p44::JsonObjectPtr varsJson(const Vars& aVars);
  void addParameter(UiNotificationBuilder::Vars& aVars, const UiNotificationParameter& aParameter);
  void flush();
};


} // namespace vdcutils
