/*
 *  Copyright (c) 2019 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of vdc-utils
 */

#pragma once

#include <type_traits>
#include <unordered_map>
#include "p44vdc/vdc_common/vdc.hpp"
#include "p44vdc/vdc_common/device.hpp"
namespace vdcutils {


template<typename T,
          typename = std::enable_if_t<std::is_base_of<p44::Device, T>::value>>
class Vdc : public p44::Vdc
{
  protected:
    std::vector<boost::intrusive_ptr<T>> mDerivedDevices;
  public:
  using p44::Vdc::Vdc;

  virtual void removeDevice(p44::DevicePtr aDevice, bool aForget = false) override
  {
    mDerivedDevices.erase(std::remove_if(mDerivedDevices.begin(), mDerivedDevices.end(), [=](auto& aDev){
       return aDevice && (aDevice->getDsUid() == aDev->getDsUid());
    }), mDerivedDevices.end());
    p44::Vdc::removeDevice(aDevice, aForget);
  }

  virtual void removeDevices(bool aForget) override
  {
    mDerivedDevices.clear();
    p44::Vdc::removeDevices(aForget);
  }

  // store specialized vDC device to avoid dynamic cast from p44::Device
  void storeDerivedDevice(boost::intrusive_ptr<T> aDevice)
  {
    if (!aDevice || devices.empty()) {
      return;
    }

    // verify if device has been added to p44::Vdc, ignore if not added
    auto added = std::any_of(devices.begin(), devices.end(), [=](auto& aDev){
      return aDevice->getDsUid() == aDev->getDsUid();
    });
    if (!added) {
      return;
    }

    // verify if device is a duplicate
    auto duplicated = std::any_of(mDerivedDevices.begin(), mDerivedDevices.end(), [=](auto& aDev){
      return aDevice->getDsUid() == aDev->getDsUid();
    });
    if (!duplicated) {
      mDerivedDevices.push_back(aDevice);
    }
  }

  using FindDevicePredicate = std::function<bool(boost::intrusive_ptr<T>)>;

  boost::intrusive_ptr<T> findDevice(FindDevicePredicate aPredicate)
  {
    auto deviceIter = std::find_if(mDerivedDevices.begin(), mDerivedDevices.end(), aPredicate);
    if (deviceIter == mDerivedDevices.end()) {
      return nullptr;
    }
    return *deviceIter;
  }

};



} // namespace vdcutils
