/*
 *  Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of vdc-utils
 */
#pragma once

#include "vdc.hpp"
#include "vdc-utils/makeintrusive.hpp"
#include "vdc-utils/property.hpp"
#include "vdc-utils/persistentstorage/persistentstoragefactory.hpp"

namespace vdcutils {

  template<typename Dev>
  class VdcWithProperties : public Vdc<Dev>
  {
    using inherited = Vdc<Dev>;

    std::vector<std::unique_ptr<PropertyBase>> properties;
    intptr_t classKey;

    template<typename T>
    auto create(const Property<T>& aProp, size_t aId)
    {
      return std::make_unique<PropertyTemplate<T>>(aProp.name, aId, classKey, aProp.readCB, aProp.writeCB);
    }




  public:
    template <typename... Args>
    VdcWithProperties(Args&&... aBaseArgs) :
      inherited(std::forward<Args>(aBaseArgs)...)
    {

    }

    template <typename... Args>
    VdcWithProperties(int aInstanceNumber, p44::VdcHost *aVdcHostP, int aTag, Property<Args>&&... aArgs) :
      inherited(aInstanceNumber, aVdcHostP, aTag)
    {
      static char key;
      classKey = OKEY(key);
      size_t i = 0;
      using expand_type = int[];
      expand_type { (properties.push_back(create(aArgs, i++)), 0)... };
    }

    template <typename Args>
    void addProperty(const Property<Args>& aProperty) {
      properties.push_back(create(aProperty, properties.size()));
    }

    virtual IPersistentStorageFactory& getStorage() {
      static auto storage = PersistentStorageFactory(this->getVdcHost().getDsParamStore());
      return storage;
    }

    virtual int numProps(int aDomain, p44::PropertyDescriptorPtr aParentDescriptor) P44_OVERRIDE
    {
      if (aParentDescriptor->isRootOfObject()) {
        return inherited::numProps(aDomain, aParentDescriptor) + properties.size();
      }
      return inherited::numProps(aDomain, aParentDescriptor);
    }

    virtual p44::PropertyDescriptorPtr getDescriptorByIndex(int aPropIndex, int aDomain, p44::PropertyDescriptorPtr aParentDescriptor) P44_OVERRIDE
    {
      if (aParentDescriptor->isRootOfObject()) {
        int n = inherited::numProps(aDomain, aParentDescriptor);
        if (aPropIndex<n)
          return inherited::getDescriptorByIndex(aPropIndex, aDomain, aParentDescriptor);
        aPropIndex -= n;
        return make_intrusive<p44::StaticPropertyDescriptor>(properties[aPropIndex]->getDescription(), aParentDescriptor);
      }

      return inherited::getDescriptorByIndex(aPropIndex, aDomain, aParentDescriptor);
    }

    virtual bool accessField(p44::PropertyAccessMode aMode, p44::ApiValuePtr aPropValue, p44::PropertyDescriptorPtr aPropertyDescriptor) P44_OVERRIDE
    {
      if (aPropertyDescriptor->hasObjectKey(classKey)) {
        auto& prop = properties[aPropertyDescriptor->fieldKey()];
        if (aMode == p44::access_read) {
          return prop->read(aPropValue);
        }
        if (aMode == p44::access_write) {
          return prop->write(aPropValue);
        }
      }

      return inherited::accessField(aMode, aPropValue, aPropertyDescriptor);

    }

  };

} // namespace vdcutils
