/*
 *  Copyright (c) 2019 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of vdc-utils
 */

#include "vdc-utils/websocket/client.hpp"
#include "vdc-utils/websocket/operation.hpp"
#include "vdc-utils/loggerext.hpp"


using namespace vdcutils::websocket;


/* ===== Client ===== */


Client::Client(p44::MainLoop& aMainLoop) :
  mQueue(aMainLoop)
{
  mQueue.isMemberVariable();
}


void Client::connect(
  std::string_view aHost,
  std::string_view aPort,
  std::string_view aTarget,
  SSLContextDecorator&& aSSLContext,
  std::shared_ptr<IEventProducer> aEventMonitor,
  p44::StatusCB aConnectedCB)
{
  LOG_WSC(LOG_NOTICE, "Connect to %s:%s%s", std::string(aHost).c_str(), std::string(aPort).c_str(), std::string(aTarget).c_str());
  mSession = std::make_shared<Session>(aHost, aPort, aTarget, std::move(aSSLContext), aEventMonitor);
  mSession->open(aConnectedCB);
}


void Client::writeData(const std::string& aData, p44::StatusCB aStatusCB)
{
  LOG_WSC(LOG_DEBUG, "Write data:\n'%s'", aData.c_str());
  auto op = vdcutils::make_intrusive<Operation>(
    *this,
    aData,
    aStatusCB
  );
  mQueue.queueOperation(op);
  mQueue.processOperations();
}


void Client::writeOperation(const std::string& aData, p44::StatusCB aStatusCB)
{
  mSession->writeData(aData, aStatusCB);
}


void Client::readData(p44::StatusCB aStatusCB)
{
  mSession->readData(
    [this, self = shared_from_this(), aStatusCB] (auto...aParams) {
      aStatusCB(aParams...);
    }
  );
}


void Client::close(p44::StatusCB aStatusCB)
{
  mSession->close(aStatusCB);
}


void Client::processOperations()
{
  mQueue.processOperations();
}
