/*
 *  Copyright (c) 2019 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of vdc-utils
 */

#pragma once

#include "vdc-utils/websocket/session.hpp"
#include "p44utils/operationqueue.hpp"


#define LOG_WSC(lvl, ...) LOG_M(lvl, "WebSocket Client", ##__VA_ARGS__)


namespace vdcutils::websocket {


/* ===== Websocket client ===== */

class IOperationClient
{
  public:
  virtual ~IOperationClient()=default;
  virtual void writeOperation(const std::string& aData, p44::StatusCB aStatusCB)=0;
  virtual void processOperations()=0;
};

class IClient
{
  public:
  virtual ~IClient() = default;
  virtual void connect(
    std::string_view aHost,
    std::string_view aPort,
    std::string_view aTarget,
    SSLContextDecorator&& aSSLContext,
    std::shared_ptr<IEventProducer> aEventMonitor,
    p44::StatusCB aConnectedCB) = 0;

  virtual void writeData(const std::string& aData, p44::StatusCB aStatusCB) = 0;
  virtual void readData(p44::StatusCB aReadCB) = 0;
  virtual void close(p44::StatusCB aStatusCB) = 0;
  virtual bool isOpen() const = 0;
};


class Client : public std::enable_shared_from_this<Client>,
               public IOperationClient,
               public IClient
{
  std::shared_ptr<Session> mSession;
  p44::OperationQueue mQueue;

  public:
  Client(p44::MainLoop& aMainLoop);
  virtual ~Client() = default;

  void connect(
    std::string_view aHost,
    std::string_view aPort,
    std::string_view aTarget,
    SSLContextDecorator&& aSSLContext,
    std::shared_ptr<IEventProducer> aEventMonitor,
    p44::StatusCB aConnectedCB);

  void writeData(const std::string& aData, p44::StatusCB aStatusCB);
  void readData(p44::StatusCB aReadCB);
  void close(p44::StatusCB aStatusCB);
  bool isOpen() const { return mSession && mSession->isOpen(); }

  protected:
  virtual void writeOperation(const std::string& aData, p44::StatusCB aStatusCB) override;
  virtual void processOperations() override;
};


} // namespace vdcutils
