/*
 *  Copyright (c) 2019 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of vdc-utils
 */

#pragma once

#include <boost/system/error_code.hpp>
#include "p44utils/error.hpp"
#include "vdc-utils/makeintrusive.hpp"
#include "vdc-utils/enumutils.hpp"


namespace vdcutils::websocket {


/* ===== WebSocket Error ===== */


struct Error : public p44::Error
{
  enum class ErrorCode {
    Resolve,
    Connect,
    Handshake,
    SSLHandshake,
    NotConnected,
    IncompleteMessage,
    CannotClose,
    OperationAborted,
    OperationRetryLimit,
    ThreadHandlerException,
    BoostSystemError,
    TerminateDataPoll
  };

  Error(ErrorCode aErrorCode);
  Error(const boost::system::error_code& aError);

  virtual const char *getErrorDomain() const { return Error::domain(); }
  static const char *domain() { return "WebSocketError"; }

  template <typename... Args>
  static auto err(Args&&... aArgs) { return vdcutils::make_intrusive<Error>(std::forward<Args>(aArgs)...); }
};


} // namespace vdcutils
