/*
 *  Copyright (c) 2020 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of vdc-utils
 */

#include "eventmonitor.hpp"
#include "p44utils/mainloop.hpp"
#include "vdc-utils/enumutils.hpp"

using namespace vdcutils::websocket;


/* ===== Event Monitor ===== */


Event EventMonitor::waitForEvent() {
  LOG_BUF(LOG_DEBUG, "Wait for signal...");
  std::unique_lock<std::mutex> lck(mMutex);
  mConditionVar.wait(lck, [this]{ return mEvent != Event::Waiting; });
  LOG_BUF(LOG_DEBUG, "Event available %d", vdcutils::toUnderlyingType(mEvent));
  return mEvent;
}


void EventMonitor::pushData(p44::ErrorPtr aError, const std::string& aData){
  std::lock_guard<std::mutex> lck(mMutex);
  mDataBuffer.emplace_back(std::make_pair(aError, aData));
  mEvent = Event::DataReady;
  mConditionVar.notify_one();
}


void EventMonitor::connected(p44::ErrorPtr aError)
{
  mConnectedError = aError;
  mEvent = Event::Connected;
  mConditionVar.notify_one();
}


void EventMonitor::writeCompleted()
{
  mEvent = Event::WriteDataCompleted;
  mConditionVar.notify_one();
}


DataList_t EventMonitor::getData()
{
  mEvent = Event::Waiting;
  DataList_t retData;
  mDataBuffer.swap(retData);
  return retData;
}


p44::ErrorPtr EventMonitor::getConnectedError()
{
  mEvent = Event::Waiting;
  p44::ErrorPtr retError;
  mConnectedError.swap(retError);
  return retError;
}


void EventMonitor::terminate()
{
  mEvent = Event::Terminated;
  mConditionVar.notify_one();
}
