/*
 *  Copyright (c) 2019 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of vdc-utils
 */

#include "vdc-utils/websocket/session.hpp"
#include "vdc-utils/websocket/error.hpp"
#include "vdc-utils/loggerext.hpp"
#include <future>

using namespace vdcutils::websocket;


/* ===== Async IOService ===== */


IOServiceHanlder::IOServiceHanlder():
  mIOService(),
  mWork(std::make_shared<boost::asio::io_service::work>(mIOService))
{
  std::thread([&]{runRoutine();}).detach();
}


void IOServiceHanlder::runRoutine()
{
  LOG_IOS(LOG_DEBUG, "Starting service routine");
  try {
    mIOService.run();
  } catch (std::exception& e) {
    LOG_IOS(LOG_ERR, "Terminating IOService. Reason '%s'", e.what());
  }
}

IOServiceHanlder::~IOServiceHanlder()
{
}


void IOServiceHanlder::stop()
{
  LOG_IOS(LOG_NOTICE, "Stop service");
  mWork.reset();
  mIOService.stop();
  LOG_IOS(LOG_NOTICE, "Stopped service");
}

