/*
 *  Copyright (c) 2019 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of vdc-utils
 */

#include "vdc-utils/websocket/operation.hpp"
#include "vdc-utils/loggerext.hpp"

using namespace vdcutils::websocket;


/* ===== Websocket operation ===== */


Operation::Operation(IOperationClient& aClient, const std::string& aJsonData, p44::StatusCB aResultHandler) :
  mClient(aClient),
  mData(aJsonData),
  mResultHandler(aResultHandler)
{
  setInitiationDelay(50*p44::MilliSecond);
}


bool Operation::initiate()
{
  if (!canInitiate(p44::MainLoop::now())) {
    return false;
  }
  // send wss request
  sendRequest();
  // mark operation as initiated
  return p44::Operation::initiate();
}


void Operation::sendRequest()
{
  mClient.writeOperation(
    mData,
    [=](auto aError) {
      mError = aError;
      mCompleted = true;
    }
  );
}


bool Operation::hasCompleted()
{
  return mCompleted;
}


p44::OperationPtr Operation::finalize()
{
  if (mResultHandler) {
    mResultHandler(mError);
    mResultHandler = {}; // call once only
  }

  mClient.processOperations();
  return p44::Operation::finalize();
}


void Operation::abortOperation(p44::ErrorPtr aError)
{
  if (!aborted && !mCompleted && mResultHandler) {
    mResultHandler(aError?aError:Error::err(Error::ErrorCode::OperationAborted));
    mResultHandler={};
  }

  p44::Operation::abortOperation(aError);
}


void Operation::restartOperation()
{
  mRetryLimit--;
  LOG_WOP(LOG_DEBUG, "Restarting operation (%d)", mRetryLimit);
  if (mRetryLimit == 0) {
    LOG_WOP(LOG_ERR, "Retry Limit exceeded -> abort operation");
    this->abortOperation(Error::err(Error::ErrorCode::OperationRetryLimit));
    return;
  }

  initiated = false;
  mCompleted = false;
  aborted = false;
  mError = {};
}
